/*
	Serial HAL for STR7
	
*/

//-----------------------------------------------------------------------------
// USART Fuction Declaration

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "stm32f10x_rcc.h"
#include "stm32f10x_usart.h"

//#ifdef _UART0_ENABLE
//------------------------------------------------------------------------------
// UART Defination
//------------------------------------------------------------------------------
//#define U0_printf(Value)					printf(Value)
//#define U0_printf							printf

#define BAUD_9600							0
#define BAUD_14400							1
#define BAUD_19200							2
#define BAUD_28800							3
#define BAUD_38400							4		
#define BAUD_57600							5		
#define BAUD_76800							6		
#define BAUD_115200							7	


#define BAUDRATE_VAL_9600					9600
#define BAUDRATE_VAL_19200					19200
#define BAUDRATE_VAL_14400					14400
#define BAUDRATE_VAL_28800					28800
#define BAUDRATE_VAL_38400					38400
#define BAUDRATE_VAL_57600					57600
#define BAUDRATE_VAL_76800					76800
#define BAUDRATE_VAL_115200					115200


#define BAUDRATE_VAL_DEFAULT				BAUDRATE_VAL_57600

#define BAUD_RATE(xtal,baud,smod)       	(256L-xtal/(baud*smod*192L))
//#define T2RELOAD(Baudrate)   				(65536-(XTAL/32/(Baudrate)))
#define T2RELOAD(Baudrate)   				(65536-((XTAL/2)/32/(Baudrate)))

#define SERIAL_MODE1                    	0x40  // SCON.6 and SCON.7
#define DOUBLE_BAUD_RATE                	0x80  // PCON.7
#define ENABLE_RECEIVE                  	0x10  // SCON.4

#define UART_RXINT_ENABLE           		BIT6
#define UART_TXINT_ENABLE           		0x40
#define UART_RX_ENABLE              		BIT6
#define UART_TX_ENABLE              		BIT4

#define UART_RX_COMPLETE            		0x01
#define UART_TX_COMPLETE            		0x02
#define UART_DATA_EMPTY             		0x20

#define TXD0_SEL_EN							BIT1
#define RXD0_SEL_EN							BIT0
#define TXD1_SEL_EN							0x40
#define RXD1_SEL_EN							0x80

#define UART_DB_BAUDRATE					0x02




#define U0R_DATA							UART0->RxBUFR
#define U0R_CSRA							US_CSR0
#define U0R_CSRB							US_CR0
#define U0R_UBRR							US_BRGR0


#define UART0_Rx_Pin (0x0001<<8)   // TQFP 64: pin N?63 , TQFP 144 pin N?143
#define UART0_Tx_Pin (0x0001<<9)   // TQFP 64: pin N?64 , TQFP 144 pin N?144

#define UART1_Rx_Pin (0x0001<<10)  // TQFP 64: pin N?1  , TQFP 144 pin N?1
#define UART1_Tx_Pin (0x0001<<11)  // TQFP 64: pin N?2  , TQFP 144 pin N?3

#ifdef _UART0_ENABLE
//------------------------------------------------------------------------------
// Serial Prot0 Utility Fuction Routine

  
#define u0_WaitForReceive()					while(USART_GetFlagStatus(USART1, USART_FLAG_RXNE) != RESET)
#define u0_RxData()		 					USART_ReceiveData(USART1)

#define _u0_TxData(Data)					USART_SendData(USART1, Data)
#define u0_WaitForSend()					while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET)//  while (!(USART1->SR & USART_FLAG_TXE))
#define u0_IsReceiveData()					(UART_FlagStatus(UART0) & UART_RxBufFull)

#define u0_EnableRxIsr()					
#define u0_EnableRx()						UART_RxConfig(UART0, ENABLE)
#define u0_EnableTx()						//UART_TxConfig(UART0, ENABLE)					

#define u0_DoubleBaud()						


void u0_baudrate(unsigned char baud);
void u0_hal_init(void);

void u0_TxData(unsigned char Data)
{
	_u0_TxData(Data);
	u0_WaitForSend();	
}

void u0_hal_init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
	

  /* Configure USART1 Rx (PA10) as input floating                             */
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  /* Configure USART1 Tx (PA9) as alternate function push-pull                */
  GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOA, &GPIO_InitStructure);


	/* Enable USART1 clocks */
	//RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART1, ENABLE);
	
	
	/* Enable the USART Transmoit interrupt: this interrupt is generated when the 
	USART1 transmit data register is empty */  
	//USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

	/* Enable the USART Receive interrupt: this interrupt is generated when the 
	USART1 receive data register is not empty */
	//USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);

	/* Enable USART1 */
	USART_Cmd(USART1, ENABLE);
}

void u0_baudrate(unsigned char baud)
{
	unsigned int baud_rate = 0;
	USART_InitTypeDef USART_InitStructure;	
	
	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	

  USART_InitStructure.USART_BaudRate            = 115200;
  USART_InitStructure.USART_WordLength          = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits            = USART_StopBits_1;
  USART_InitStructure.USART_Parity              = USART_Parity_No ;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;
  USART_InitStructure.USART_Clock               = USART_Clock_Disable;
  USART_InitStructure.USART_CPOL                = USART_CPOL_Low;
  USART_InitStructure.USART_CPHA                = USART_CPHA_2Edge;
  USART_InitStructure.USART_LastBit             = USART_LastBit_Disable;
  USART_Init(USART1, &USART_InitStructure);
}


void U0_Puts(char *str)
{
    while(*str != 0)
    {
        u0_TxData(*str);
        str++;
    }
}

void U0_printf(char *fmt,...)
{
    va_list ap;
    char string[128];

    va_start(ap,fmt);
    vsprintf(string,fmt,ap);
    U0_Puts(string);
    va_end(ap);
}
//------------------------------------------------------------------------------
#endif	//end _UART0_ENABLE


#ifdef _UART1_ENABLE
//------------------------------------------------------------------------------


// Serial Prot1 Utility Fuction Routine
#define u1_WaitForReceive()					while(!(UCSR1A & UART_RX_COMPLETE))
#define u1_RxData()		 					(UDR1)

#define _u1_TxData(Data)					(UDR1 = Data)
#define u1_WaitForSend()					while(!(UCSR1A & UART_DATA_EMPTY))

#define u1_IsReceiveData()					((UCSR1A & UART_RX_COMPLETE))

#define u1_EnableRxIsr()					(UCSR1B |= (UART_RXINT_ENABLE|UART_RX_ENABLE)) 
	
void u1_baudrate(unsigned char baud);
void u1_hal_init(void);

void u1_TxData(unsigned char Data)
{
	u1_WaitForSend();
	_u1_TxData(Data);
}



void u1_hal_init(void)
{

}

void u1_baudrate(unsigned char baud)
{
	switch (baud)
	{
		case BAUD_9600:
			UBRR1L = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			UBRR1L = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			UBRR1L = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			UBRR1L = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			UBRR1L = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			UBRR1L = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			UBRR1L = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			UBRR1L = BAUDRATE_VAL_115200;
			break;		
		default :
			UBRR1L = BAUDRATE_VAL_DEFAULT;			
			break;
	}	
}


//------------------------------------------------------------------------------
#endif


#ifdef _UART2_ENABLE
//------------------------------------------------------------------------------
// Serial Prot1 Utility Fuction Routine
#define U2R_DATA							UART2->RxBUFR
#define U2R_CSRA							US_CSR2
#define U2R_CSRB							US_CR2
#define U2R_UBRR							US_BRGR2

#define UART2_Rx_Pin 						(0x0001<<13)
#define UART2_Tx_Pin 						(0x0001<<14)

#define u2_WaitForReceive()					while(!(UART_FlagStatus(UART2) & UART_RxBufFull))
#define u2_RxData()		 					(U2R_DATA)

#define _u2_TxData(Data)					(UART2->TxBUFR = Data)
#define u2_WaitForSend()					while(!((UART_FlagStatus(UART2)) & UART_TxEmpty))

#define u2_IsReceiveData()					(UART_FlagStatus(UART2) & UART_RxBufFull)

#define u2_EnableRxIsr()					(UCSR1B |= (UART_RXINT_ENABLE|UART_RX_ENABLE)) 
#define u2_EnableRx()						UART_RxConfig(UART2 ,ENABLE)
#define u2_EnableTx()						//UART_TxConfig(UART2 ,ENABLE)

void u2_baudrate(unsigned char baud);
void u2_hal_init(void);

void u2_TxData(unsigned char Data)
{
	_u2_TxData(Data);
	u2_WaitForSend();
}

void u2_hal_init(void)
{
//  Configure the GPIO pins 
	GPIO_Config(GPIO0, UART2_Tx_Pin, GPIO_AF_PP);
  	GPIO_Config(GPIO0, UART2_Rx_Pin, GPIO_IN_TRI_CMOS);

 	// UART peripheral configuration -------------------------------------------
   	//  Configure the UART X 
  	UART_OnOffConfig(UART2, ENABLE);      
  	UART_FifoConfig (UART2, DISABLE);     
  	UART_FifoReset  (UART2 , UART_RxFIFO);
  	UART_FifoReset  (UART2 , UART_TxFIFO);
  	UART_LoopBackConfig(UART2 , DISABLE); 
	
	// enable serial uart & receiver
	u2_EnableTx();
	u2_EnableRx(); 
	
  	// Configure the EIC channel interrupt 
  	EIC_IRQChannelPriorityConfig(UART2_IRQChannel, 1);
  	EIC_IRQChannelConfig(UART2_IRQChannel, ENABLE);
}

void u2_baudrate(unsigned char baud)
{
	unsigned int baud_rate = 0;
	
	switch (baud)
	{
		case BAUD_9600:
			baud_rate = BAUDRATE_VAL_9600;
			break;
		case BAUD_14400:
			baud_rate = BAUDRATE_VAL_14400;
			break;		
		case BAUD_19200:
			baud_rate = BAUDRATE_VAL_19200;
			break;		
		case BAUD_28800:
			baud_rate = BAUDRATE_VAL_28800;
			break;		
		case BAUD_38400:
			baud_rate = BAUDRATE_VAL_38400;
			break;		
		case BAUD_57600:
			baud_rate = BAUDRATE_VAL_57600;
			break;		
		case BAUD_76800:
			baud_rate = BAUDRATE_VAL_76800;
			break;		
		case BAUD_115200:
			baud_rate = BAUDRATE_VAL_115200;
			break;		
		default :
			baud_rate = BAUDRATE_VAL_DEFAULT;			
			break;
	}	

	UART_Config(UART2, baud_rate,UART_NO_PARITY,UART_1_StopBits,UARTM_8D);
}

void U2_Puts(char *str)
{
    while(*str != 0)
    {
        u2_TxData(*str);
        str++;
    }
}

void U2_printf(char *fmt,...)
{
    va_list ap;
    char string[128];

    va_start(ap,fmt);
    vsprintf(string,fmt,ap);
    U2_Puts(string);
    va_end(ap);
}
//------------------------------------------------------------------------------
#endif